require "tiedye/version"

module Tiedye
  def self.to_rgb(hex)
    hex = hex.gsub("#", "")
    hex = [hex[0..1], hex[2..3], hex[4..5]]
    options = {"FF" => 255, "00" => 0}
    return hex.collect{|h| options[h]}
    # hex = to_6_hex(hex)
    # hex_pairs = hex.scan(/.{1,2}/)
    # rgb = []
    # rgb[0] = hex_pair_to_rgb_digit(hex_pairs[0])
    # rgb[1] = hex_pair_to_rgb_digit(hex_pairs[1])
    # rgb[2] = hex_pair_to_rgb_digit(hex_pairs[2])
    # return rgb
  end

  def self.to_hex(r = "x", g = "x", b = "x")
    if r.class == Array
      temp = r
      r = temp[0]
      g = temp[1]
      b = temp[2]
    end
    return random_hex if invalid(r, g, b)
    r, g, b = rgb_float_to_int(r, g, b)
    return "#" + hex_digit(r) + hex_digit(g) + hex_digit(b)
  end

  def self.method_missing(name, *args)
    return COLORS[name.to_s.downcase] if COLORS[name.to_s.downcase]
    return COLORS[name.to_s] if COLORS[name.to_s]
    return name
  end

  def self.all_colors
    return COLORS.keys
  end

private
  def self.hex_pair_to_rgb_digit(hex_pair)
    hexz = hex_pair.upcase.split("")
    number  = HEX_DIGIT.key(hexz[0]) * 16
    number += HEX_DIGIT.key(hexz[1])
    return number
  end

  def self.to_6_hex(hex)
    hex = hex.gsub("#", "")
    if hex.length == 3
      letters = hex.split("")
      hex = ""
      letters.each do |letter|
        hex += letter + letter
      end
    end
    return hex
  end

  def self.random_hex
    hex = "#"
    3.times do
      hex += hex_digit(rand(0..255))
    end
    return hex
  end

  def self.hex_digit(digit)
    return HEX_DIGIT[digit / 16] + HEX_DIGIT[digit % 16]
  end

  def self.rgb_float_to_int(r, g, b)
    if r.class == Float && r <= 1 && g <= 1 && b <= 1
      r = (r * 255).to_i
      g = (g * 255).to_i
      b = (b * 255).to_i
    end
    return [r, g, b]
  end

  def self.invalid(r, g, b)
    begin
      [r, g, b].each do |x|
        return true if x == "x"
        return true if x > 255 || x < 0
      end
    rescue
      return false
    end
    return false
  end

  HEX_DIGIT = {
    0  => "0", 1  => "1", 2  => "2", 3  => "3", 4  => "4",
    5  => "5", 6  => "6", 7  => "7", 8  => "8", 9  => "9",
    10 => "A", 11 => "B", 12 => "C", 13 => "D", 14 => "E", 15 => "F",
  }

  COLORS = {
    "alice_blue" => "#f0f8ff",
    "antique_white" => "#faebd7",
    "antique_white" => "#ffefdb",
    "antique_white2" => "#eedfcc",
    "antique_white3" => "#cdc0b0",
    "antique_white4" => "#8b8378",
    "aquamarine" => "#7fffd4",
    "aquamarine2" => "#76eec6",
    "aquamarine4" => "#458b74",
    "azure" => "#f0ffff",
    "azure2" => "#e0eeee",
    "azure3" => "#c1cdcd",
    "azure4" => "#838b8b",
    "beige" => "#f5f5dc",
    "bisque" => "#ffe4c4",
    "bisque2" => "#eed5b7",
    "bisque3" => "#cdb79e",
    "bisque4" => "#8b7d6b",
    "black" => "#000000",
    "blanched_almond" => "#ffebcd",
    "blue" => "#0000ff",
    "blue2" => "#0000ee",
    "blue4" => "#00008b",
    "blue_violet" => "#8a2be2",
    "brown" => "#a52a2a",
    "brown" => "#ff4040",
    "brown2" => "#ee3b3b",
    "brown3" => "#cd3333",
    "brown4" => "#8b2323",
    "burlywood" => "#deb887",
    "burlywood" => "#ffd39b",
    "burlywood2" => "#eec591",
    "burlywood3" => "#cdaa7d",
    "burlywood4" => "#8b7355",
    "cadet_blue" => "#5f9ea0",
    "cadet_blue" => "#98f5ff",
    "cadet_blue2" => "#8ee5ee",
    "cadet_blue3" => "#7ac5cd",
    "cadet_blue4" => "#53868b",
    "chartreuse" => "#7fff00",
    "chartreuse2" => "#76ee00",
    "chartreuse3" => "#66cd00",
    "chartreuse4" => "#458b00",
    "chocolate" => "#d2691e",
    "chocolate" => "#ff7f24",
    "chocolate2" => "#ee7621",
    "chocolate3" => "#cd661d",
    "coral" => "#ff7f50",
    "coral" => "#ff7256",
    "coral2" => "#ee6a50",
    "coral3" => "#cd5b45",
    "coral4" => "#8b3e2f",
    "cornflower_blue" => "#6495ed",
    "cornsilk" => "#fff8dc",
    "cornsilk2" => "#eee8cd",
    "cornsilk3" => "#cdc8b1",
    "cornsilk4" => "#8b8878",
    "cyan" => "#00ffff",
    "cyan2" => "#00eeee",
    "cyan3" => "#00cdcd",
    "cyan4" => "#008b8b",
    "dark_goldenrod" => "#b8860b",
    "dark_goldenrod" => "#ffb90f",
    "dark_goldenrod2" => "#eead0e",
    "dark_goldenrod3" => "#cd950c",
    "dark_goldenrod4" => "#8b6508",
    "dark_green" => "#006400",
    "dark_khaki" => "#bdb76b",
    "dark_oliveGreen" => "#556b2f",
    "dark_oliveGreen" => "#caff70",
    "dark_oliveGreen2" => "#bcee68",
    "dark_oliveGreen3" => "#a2cd5a",
    "dark_oliveGreen4" => "#6e8b3d",
    "dark_orange" => "#ff8c00",
    "dark_orange" => "#ff7f00",
    "dark_orange2" => "#ee7600",
    "dark_orange3" => "#cd6600",
    "dark_orange4" => "#8b4500",
    "dark_orchid" => "#9932cc",
    "dark_orchid" => "#bf3eff",
    "dark_orchid2" => "#b23aee",
    "dark_orchid3" => "#9a32cd",
    "dark_orchid4" => "#68228b",
    "dark_salmon" => "#e9967a",
    "dark_seaGreen" => "#8fbc8f",
    "dark_seaGreen" => "#c1ffc1",
    "dark_seaGreen2" => "#b4eeb4",
    "dark_seaGreen3" => "#9bcd9b",
    "dark_seaGreen4" => "#698b69",
    "dark_slateBlue" => "#483d8b",
    "dark_slateGray" => "#2f4f4f",
    "dark_slateGray" => "#97ffff",
    "dark_slateGray2" => "#8deeee",
    "dark_slateGray3" => "#79cdcd",
    "dark_slateGray4" => "#528b8b",
    "dark_turquoise" => "#00ced1",
    "dark_violet" => "#9400d3",
    "deep_pink" => "#ff1493",
    "deep_pink2" => "#ee1289",
    "deep_pink3" => "#cd1076",
    "deep_pink4" => "#8b0a50",
    "deep_skyBlue" => "#00bfff",
    "deep_skyBlue2" => "#00b2ee",
    "deep_skyBlue3" => "#009acd",
    "deep_skyBlue4" => "#00688b",
    "dim_gray" => "#696969",
    "dodger_blue" => "#1e90ff",
    "dodger_blue2" => "#1c86ee",
    "dodger_blue3" => "#1874cd",
    "dodger_blue4" => "#104e8b",
    "firebrick" => "#b22222",
    "firebrick" => "#ff3030",
    "firebrick2" => "#ee2c2c",
    "firebrick3" => "#cd2626",
    "firebrick4" => "#8b1a1a",
    "floral_white" => "#fffaf0",
    "forest_green" => "#228b22",
    "gainsboro" => "#dcdcdc",
    "ghost_white" => "#f8f8ff",
    "gold" => "#ffd700",
    "gold2" => "#eec900",
    "gold3" => "#cdad00",
    "gold4" => "#8b7500",
    "goldenrod" => "#daa520",
    "goldenrod" => "#ffc125",
    "goldenrod2" => "#eeb422",
    "goldenrod3" => "#cd9b1d",
    "goldenrod4" => "#8b6914",
    "gray" => "#bebebe",
    "gray" => "#030303",
    "gray10" => "#1a1a1a",
    "gray11" => "#1c1c1c",
    "gray12" => "#1f1f1f",
    "gray13" => "#212121",
    "gray14" => "#242424",
    "gray15" => "#262626",
    "gray16" => "#292929",
    "gray17" => "#2b2b2b",
    "gray18" => "#2e2e2e",
    "gray19" => "#303030",
    "gray2" => "#050505",
    "gray20" => "#333333",
    "gray21" => "#363636",
    "gray22" => "#383838",
    "gray23" => "#3b3b3b",
    "gray24" => "#3d3d3d",
    "gray25" => "#404040",
    "gray26" => "#424242",
    "gray27" => "#454545",
    "gray28" => "#474747",
    "gray29" => "#4a4a4a",
    "gray3" => "#080808",
    "gray30" => "#4d4d4d",
    "gray31" => "#4f4f4f",
    "gray32" => "#525252",
    "gray33" => "#545454",
    "gray34" => "#575757",
    "gray35" => "#595959",
    "gray36" => "#5c5c5c",
    "gray37" => "#5e5e5e",
    "gray38" => "#616161",
    "gray39" => "#636363",
    "gray4" => "#0a0a0a",
    "gray40" => "#666666",
    "gray41" => "#696969",
    "gray42" => "#6b6b6b",
    "gray43" => "#6e6e6e",
    "gray44" => "#707070",
    "gray45" => "#737373",
    "gray46" => "#757575",
    "gray47" => "#787878",
    "gray48" => "#7a7a7a",
    "gray49" => "#7d7d7d",
    "gray5" => "#0d0d0d",
    "gray50" => "#7f7f7f",
    "gray51" => "#828282",
    "gray52" => "#858585",
    "gray53" => "#878787",
    "gray54" => "#8a8a8a",
    "gray55" => "#8c8c8c",
    "gray56" => "#8f8f8f",
    "gray57" => "#919191",
    "gray58" => "#949494",
    "gray59" => "#969696",
    "gray6" => "#0f0f0f",
    "gray60" => "#999999",
    "gray61" => "#9c9c9c",
    "gray62" => "#9e9e9e",
    "gray63" => "#a1a1a1",
    "gray64" => "#a3a3a3",
    "gray65" => "#a6a6a6",
    "gray66" => "#a8a8a8",
    "gray67" => "#ababab",
    "gray68" => "#adadad",
    "gray69" => "#b0b0b0",
    "gray7" => "#121212",
    "gray70" => "#b3b3b3",
    "gray71" => "#b5b5b5",
    "gray72" => "#b8b8b8",
    "gray73" => "#bababa",
    "gray74" => "#bdbdbd",
    "gray75" => "#bfbfbf",
    "gray76" => "#c2c2c2",
    "gray77" => "#c4c4c4",
    "gray78" => "#c7c7c7",
    "gray79" => "#c9c9c9",
    "gray8" => "#141414",
    "gray80" => "#cccccc",
    "gray81" => "#cfcfcf",
    "gray82" => "#d1d1d1",
    "gray83" => "#d4d4d4",
    "gray84" => "#d6d6d6",
    "gray85" => "#d9d9d9",
    "gray86" => "#dbdbdb",
    "gray87" => "#dedede",
    "gray88" => "#e0e0e0",
    "gray89" => "#e3e3e3",
    "gray9" => "#171717",
    "gray90" => "#e5e5e5",
    "gray91" => "#e8e8e8",
    "gray92" => "#ebebeb",
    "gray93" => "#ededed",
    "gray94" => "#f0f0f0",
    "gray95" => "#f2f2f2",
    "gray97" => "#f7f7f7",
    "gray98" => "#fafafa",
    "gray99" => "#fcfcfc",
    "green" => "#00ff00",
    "green2" => "#00ee00",
    "green3" => "#00cd00",
    "green4" => "#008b00",
    "green_yellow" => "#adff2f",
    "honeydew" => "#f0fff0",
    "honeydew2" => "#e0eee0",
    "honeydew3" => "#c1cdc1",
    "honeydew4" => "#838b83",
    "hot_pink" => "#ff69b4",
    "hot_pink" => "#ff6eb4",
    "hot_pink2" => "#ee6aa7",
    "hot_pink3" => "#cd6090",
    "hot_pink4" => "#8b3a62",
    "indian_red" => "#cd5c5c",
    "indian_red" => "#ff6a6a",
    "indian_red2" => "#ee6363",
    "indian_red3" => "#cd5555",
    "indian_red4" => "#8b3a3a",
    "ivory" => "#fffff0",
    "ivory2" => "#eeeee0",
    "ivory3" => "#cdcdc1",
    "ivory4" => "#8b8b83",
    "khaki" => "#f0e68c",
    "khaki2" => "#fff68f",
    "khaki3" => "#eee685",
    "khaki4" => "#cdc673",
    "khaki5" => "#8b864e",
    "lavender" => "#e6e6fa",
    "lavender_blush" => "#fff0f5",
    "lavender_blush2" => "#eee0e5",
    "lavender_blush3" => "#cdc1c5",
    "lavender_blush4" => "#8b8386",
    "lawn_green" => "#7cfc00",
    "lemon_chiffon" => "#fffacd",
    "lemon_chiffon2" => "#eee9bf",
    "lemon_chiffon3" => "#cdc9a5",
    "lemon_chiffon4" => "#8b8970",
    "light" => "#eedd82",
    "light_blue" => "#add8e6",
    "light_blue2" => "#bfefff",
    "light_blue3" => "#b2dfee",
    "light_blue4" => "#9ac0cd",
    "light_blue5" => "#68838b",
    "light_coral" => "#f08080",
    "light_cyan" => "#e0ffff",
    "light_cyan2" => "#d1eeee",
    "light_cyan3" => "#b4cdcd",
    "light_cyan4" => "#7a8b8b",
    "light_goldenrod" => "#ffec8b",
    "light_goldenrod2" => "#eedc82",
    "light_goldenrod3" => "#cdbe70",
    "light_goldenrod4" => "#8b814c",
    "light_goldenrodYellow" => "#fafad2",
    "light_gray" => "#d3d3d3",
    "light_pink" => "#ffb6c1",
    "light_pink2" => "#ffaeb9",
    "light_pink3" => "#eea2ad",
    "light_pink4" => "#cd8c95",
    "light_pink5" => "#8b5f65",
    "light_salmon" => "#ffa07a",
    "light_salmon2" => "#ee9572",
    "light_salmon3" => "#cd8162",
    "light_salmon4" => "#8b5742",
    "light_seaGreen" => "#20b2aa",
    "light_skyBlue" => "#87cefa",
    "light_skyBlue2" => "#b0e2ff",
    "light_skyBlue3" => "#a4d3ee",
    "light_skyBlue4" => "#8db6cd",
    "light_skyBlue5" => "#607b8b",
    "light_slateBlue" => "#8470ff",
    "light_slateGray" => "#778899",
    "light_steelBlue" => "#b0c4de",
    "light_steelBlue2" => "#cae1ff",
    "light_steelBlue3" => "#bcd2ee",
    "light_steelBlue4" => "#a2b5cd",
    "light_steelBlue5" => "#6e7b8b",
    "light_yellow" => "#ffffe0",
    "light_yellow2" => "#eeeed1",
    "light_yellow3" => "#cdcdb4",
    "light_yellow4" => "#8b8b7a",
    "lime_green" => "#32cd32",
    "linen" => "#faf0e6",
    "magenta" => "#ff00ff",
    "magenta2" => "#ee00ee",
    "magenta3" => "#cd00cd",
    "magenta4" => "#8b008b",
    "maroon" => "#b03060",
    "maroon2" => "#ff34b3",
    "maroon3" => "#ee30a7",
    "maroon4" => "#cd2990",
    "maroon5" => "#8b1c62",
    "medium" => "#66cdaa",
    "medium_aquamarine" => "#66cdaa",
    "medium_blue" => "#0000cd",
    "medium_orchid" => "#ba55d3",
    "medium_orchid2" => "#e066ff",
    "medium_orchid3" => "#d15fee",
    "medium_orchid4" => "#b452cd",
    "medium_orchid5" => "#7a378b",
    "medium_purple" => "#9370db",
    "medium_purple2" => "#ab82ff",
    "medium_purple3" => "#9f79ee",
    "medium_purple4" => "#8968cd",
    "medium_purple5" => "#5d478b",
    "medium_seaGreen" => "#3cb371",
    "medium_slateBlue" => "#7b68ee",
    "medium_springGreen" => "#00fa9a",
    "medium_turquoise" => "#48d1cc",
    "medium_violetRed" => "#c71585",
    "midnight_blue" => "#191970",
    "mint_cream" => "#f5fffa",
    "misty_rose" => "#ffe4e1",
    "misty_rose2" => "#eed5d2",
    "misty_rose3" => "#cdb7b5",
    "misty_rose4" => "#8b7d7b",
    "moccasin" => "#ffe4b5",
    "navajo_white" => "#ffdead",
    "navajo_white2" => "#eecfa1",
    "navajo_white3" => "#cdb38b",
    "navajo_white4" => "#8b795e",
    "navy_blue" => "#000080",
    "old_lace" => "#fdf5e6",
    "olive_drab" => "#6b8e23",
    "olive_drab2" => "#c0ff3e",
    "olive_drab3" => "#b3ee3a",
    "olive_drab4" => "#698b22",
    "orange" => "#ffa500",
    "orange2" => "#ee9a00",
    "orange3" => "#cd8500",
    "orange4" => "#8b5a00",
    "orange_red" => "#ff4500",
    "orange_red2" => "#ee4000",
    "orange_red3" => "#cd3700",
    "orange_red4" => "#8b2500",
    "orchid" => "#da70d6",
    "orchid2" => "#ff83fa",
    "orchid3" => "#ee7ae9",
    "orchid4" => "#cd69c9",
    "orchid5" => "#8b4789",
    "pale" => "#db7093",
    "pale_goldenrod" => "#eee8aa",
    "pale_green" => "#98fb98",
    "pale_green2" => "#9aff9a",
    "pale_green3" => "#90ee90",
    "pale_green4" => "#7ccd7c",
    "pale_green5" => "#548b54",
    "pale_turquoise" => "#afeeee",
    "pale_turquoise2" => "#bbffff",
    "pale_turquoise3" => "#aeeeee",
    "pale_turquoise4" => "#96cdcd",
    "pale_turquoise5" => "#668b8b",
    "pale_violetRed" => "#db7093",
    "pale_violetRed2" => "#ff82ab",
    "pale_violetRed3" => "#ee799f",
    "pale_violetRed4" => "#cd6889",
    "pale_violetRed5" => "#8b475d",
    "papaya_whip" => "#ffefd5",
    "peach_puff" => "#ffdab9",
    "peach_puff2" => "#eecbad",
    "peach_puff3" => "#cdaf95",
    "peach_puff4" => "#8b7765",
    "pink" => "#ffc0cb",
    "pink2" => "#ffb5c5",
    "pink3" => "#eea9b8",
    "pink4" => "#cd919e",
    "pink5" => "#8b636c",
    "plum" => "#dda0dd",
    "plum2" => "#ffbbff",
    "plum3" => "#eeaeee",
    "plum4" => "#cd96cd",
    "plum5" => "#8b668b",
    "powder_blue" => "#b0e0e6",
    "purple" => "#a020f0",
    "purple2" => "#9b30ff",
    "purple3" => "#912cee",
    "purple4" => "#7d26cd",
    "purple5" => "#551a8b",
    "red" => "#ff0000",
    "red2" => "#ee0000",
    "red3" => "#cd0000",
    "red4" => "#8b0000",
    "rosy_brown" => "#bc8f8f",
    "rosy_brown2" => "#ffc1c1",
    "rosy_brown3" => "#eeb4b4",
    "rosy_brown4" => "#cd9b9b",
    "rosy_brown5" => "#8b6969",
    "royal_blue" => "#4169e1",
    "royal_blue2" => "#4876ff",
    "royal_blue3" => "#436eee",
    "royal_blue4" => "#3a5fcd",
    "royal_blue5" => "#27408b",
    "saddle_brown" => "#8b4513",
    "salmon" => "#fa8072",
    "salmon2" => "#ff8c69",
    "salmon3" => "#ee8262",
    "salmon4" => "#cd7054",
    "salmon5" => "#8b4c39",
    "sandy_brown" => "#f4a460",
    "sea_green" => "#54ff9f",
    "sea_green2" => "#4eee94",
    "sea_green3" => "#43cd80",
    "sea_green4" => "#2e8b57",
    "seashell" => "#fff5ee",
    "seashell2" => "#eee5de",
    "seashell3" => "#cdc5bf",
    "seashell4" => "#8b8682",
    "sienna" => "#a0522d",
    "sienna2" => "#ff8247",
    "sienna3" => "#ee7942",
    "sienna4" => "#cd6839",
    "sienna5" => "#8b4726",
    "sky_blue" => "#87ceeb",
    "sky_blue2" => "#87ceff",
    "sky_blue3" => "#7ec0ee",
    "sky_blue4" => "#6ca6cd",
    "sky_blue5" => "#4a708b",
    "slate_blue" => "#6a5acd",
    "slate_blue2" => "#836fff",
    "slate_blue3" => "#7a67ee",
    "slate_blue4" => "#6959cd",
    "slate_blue5" => "#473c8b",
    "slate_gray" => "#708090",
    "slate_gray2" => "#c6e2ff",
    "slate_gray4" => "#b9d3ee",
    "slate_gray5" => "#9fb6cd",
    "slate_gray6" => "#6c7b8b",
    "snow" => "#fffafa",
    "snow2" => "#eee9e9",
    "snow3" => "#cdc9c9",
    "snow4" => "#8b8989",
    "spring_green" => "#00ff7f",
    "spring_green2" => "#00ee76",
    "spring_green3" => "#00cd66",
    "spring_green4" => "#008b45",
    "steel_blue" => "#4682b4",
    "steel_blue2" => "#63b8ff",
    "steel_blue3" => "#5cacee",
    "steel_blue4" => "#4f94cd",
    "steel_blue5" => "#36648b",
    "tan" => "#d2b48c",
    "tan2" => "#ffa54f",
    "tan3" => "#ee9a49",
    "tan4" => "#cd853f",
    "tan5" => "#8b5a2b",
    "thistle" => "#d8bfd8",
    "thistle2" => "#ffe1ff",
    "thistle3" => "#eed2ee",
    "thistle4" => "#cdb5cd",
    "thistle5" => "#8b7b8b",
    "tomato" => "#ff6347",
    "tomato2" => "#ee5c42",
    "tomato3" => "#cd4f39",
    "tomato4" => "#8b3626",
    "turquoise" => "#40e0d0",
    "turquoise2" => "#00f5ff",
    "turquoise3" => "#00e5ee",
    "turquoise4" => "#00c5cd",
    "turquoise5" => "#00868b",
    "violet" => "#ee82ee",
    "violet_red" => "#d02090",
    "violet_red2" => "#ff3e96",
    "violet_red3" => "#ee3a8c",
    "violet_red4" => "#cd3278",
    "violet_red5" => "#8b2252",
    "wheat" => "#f5deb3",
    "wheat2" => "#ffe7ba",
    "wheat3" => "#eed8ae",
    "wheat4" => "#cdba96",
    "wheat5" => "#8b7e66",
    "white" => "#ffffff",
    "white_smoke" => "#f5f5f5",
    "yellow" => "#ffff00",
    "yellow2" => "#eeee00",
    "yellow3" => "#cdcd00",
    "yellow4" => "#8b8b00",
    "yellow_green" => "#9acd32"}
end








